Attribute VB_Name = "CD_Colontitles"
Option Private Module
Option Explicit

Public Function CreateColontitles(target As Word.Document, docMeta As InfoDocument, _
                                  props As ItemColontitles, incrementCallback$)
  Dim sPrevious$
  Dim nCurrent&: nCurrent = props.start_
  Do While nCurrent <= props.finish_
    Dim aSection As Word.Section: Set aSection = target.Sections(nCurrent)
    Dim headRange As Word.Range: Set headRange = aSection.Range.Paragraphs.First.Range
    If Not (IsHeader(headRange, 1) Or IsToCHeader(headRange)) Or VBA.Len(headRange.Text) <= 3 Then
      Call ResetSection(aSection)
      GoTo NEXT_SECTION
    End If
    
   Call ProcessSection(aSection, docMeta, props, sPrevious, nCurrent = props.start_)
    
NEXT_SECTION:
    nCurrent = nCurrent + 1
    Call CallByName(CSE_ProgressBar, incrementCallback, VbMethod)
  Loop
End Function

' ============
Private Function ResetSection(aSection As Word.Section)
  aSection.PageSetup.DifferentFirstPageHeaderFooter = False
  aSection.PageSetup.OddAndEvenPagesHeaderFooter = True
  aSection.Headers(wdHeaderFooterPrimary).LinkToPrevious = True
  aSection.Headers(wdHeaderFooterEvenPages).LinkToPrevious = True
  aSection.Footers(wdHeaderFooterPrimary).LinkToPrevious = True
  aSection.Footers(wdHeaderFooterEvenPages).LinkToPrevious = True
End Function

Private Function ScanSection(target As Word.Section, ByRef sPrevious$) As SectionData
  Dim aData As SectionData
  
  Dim headRange As Word.Range: Set headRange = target.Range.Paragraphs.First.Range
  If headRange.Style = " " Then
    Call headRange.MoveEnd(wdWord, -1)
    aData.sChapter = headRange.Text
  Else
    aData.sName = headRange.Text
    If IsUsualSecName(aData.sName, headRange) Then _
      aData.sName = " " & aData.sName
    If IsToCHeader(aData.sName) Then _
      aData.sChapter = aData.sName
  End If
  
  aData.bNewChapter = aData.sChapter = sPrevious And Not sPrevious = ""
  sPrevious = aData.sChapter
  ScanSection = aData
End Function

Private Function ProcessSection(target As Word.Section, docMeta As InfoDocument, props As ItemColontitles, ByRef sPrevious$, bFrist As Boolean)
  Dim sData As SectionData: sData = ScanSection(target, sPrevious)
  Dim cPos As ColontitlePosition
  
  If props.doTopLeft Then
    cPos.top_ = True
    cPos.left_ = True
    cPos.source_ = props.mTopLeft
    Call ProcessTitle(target, docMeta, cPos, sData, Not bFrist And DoLink(cPos.source_, sData.bNewChapter))
  End If
  
  If props.doTopRight Then
    cPos.top_ = True
    cPos.left_ = False
    cPos.source_ = props.mTopRight
    Call ProcessTitle(target, docMeta, cPos, sData, Not bFrist And DoLink(cPos.source_, sData.bNewChapter))
  End If
  
  If props.doBottomLeft Then
    cPos.top_ = False
    cPos.left_ = True
    cPos.source_ = props.mBottomLeft
    Call ProcessTitle(target, docMeta, cPos, sData, Not bFrist And DoLink(cPos.source_, sData.bNewChapter))
  End If
  
  If props.doBottomRight Then
    cPos.top_ = False
    cPos.left_ = False
    cPos.source_ = props.mBottomRight
    Call ProcessTitle(target, docMeta, cPos, sData, Not bFrist And DoLink(cPos.source_, sData.bNewChapter))
  End If
End Function

Private Function ProcessTitle(target As Word.Section, docMeta As InfoDocument, cPos As ColontitlePosition, sData As SectionData, linkToPrev As Boolean)
  Dim oTitle As Word.HeaderFooter: Set oTitle = GetHeader(target, cPos)
  oTitle.LinkToPrevious = linkToPrev
  If linkToPrev Then _
    Exit Function

  Call ReplaceColontitle(oTitle, cPos, TitlesText(cPos.source_, sData, docMeta))
End Function

Private Function TitlesText(sMode As TSource, sData As SectionData, docMeta As InfoDocument) As String
  Select Case sMode:
  Case T_SOURCE_BOOK: TitlesText = docMeta.BookText
  Case T_SOURCE_VOLUME: TitlesText = docMeta.VolumeText
  Case T_SOURCE_DOCUMENT: TitlesText = docMeta.document_
  Case T_SOURCE_SECTION: TitlesText = sData.sName
  Case T_SOURCE_CHAPTER:  TitlesText = sData.sChapter
  Case T_SOURCE_CONCEPT:
    TitlesText = "     """"" & ", " & CStr(Year(Date))
  End Select
End Function

Private Function ReplaceColontitle(target As Word.HeaderFooter, cPos As ColontitlePosition, sText$)
  If cPos.top_ Then
    Call InsertHeader(target.Range, sText, cPos)
    If cPos.source_ <> T_SOURCE_DOCUMENT And cPos.source_ <> T_SOURCE_CONCEPT Then _
      Call BoldHeaderPart(target.Range)
    Call HeadersWrap(target.Range)
  Else
    If cPos.source_ = T_SOURCE_SECTION And InStr(UCase(sText), UCase("")) = 0 Then _
      sText = Mid(sText, InStr(sText, ".") + 1)
    Call InsertFooter(target.Range, sText, cPos)
  End If
End Function

Private Function BoldHeaderPart(target As Word.Range)
  target.Bold = False
  Dim boldPart As Word.Range:  Set boldPart = target.Words.First
  If boldPart.MoveEndUntil(".", wdForward) Then
    boldPart.End = boldPart.End + 1
    boldPart.Bold = True
  End If
End Function

Private Function InsertHeader(target As Word.Range, sText$, cPos As ColontitlePosition)
  target.Text = sText
  target.Style = " "
  Call target.Words.Last.Delete
End Function

Private Function InsertFooter(target As Word.Range, sText$, cPos As ColontitlePosition)
  Call CleanFooter(target, cPos)
  
  If Len(sText) >= 2 Then
    If Asc(Left(sText, 1)) < 33 Then sText = Mid(sText, 2)
    If Asc(Right(sText, 1)) < 33 Then sText = Left(sText, Len(sText) - 1)
  End If
  
  Call target.InsertBefore(UCase(sText))
  If cPos.left_ Then
    Call FormatLeftFooter(target)
  Else
    Call FormatRightFooter(target)
  End If
End Function

Private Function FormatRightFooter(target As Word.Range)
  Dim rDest As Word.Range: Set rDest = target.Duplicate
  Call rDest.MoveEnd(wdWord, -2)
  
  If Len(rDest.Text) < 30 Then
    Call rDest.InsertBefore(Chr(11))
  Else
    Dim rSecondLine As Word.Range: Set rSecondLine = rDest.Duplicate
    Call rSecondLine.Collapse(wdCollapseEnd)
    Do
      Call rDest.MoveEnd(wdWord, -1)
      Call rSecondLine.MoveStart(wdWord, -1)
    Loop Until Len(rDest.Text) + 2 < Len(rSecondLine.Text)
    
    Call rSecondLine.InsertBefore(Chr(11))
    Call target.Words.Last.Delete
  End If
End Function

Private Function FormatLeftFooter(target As Word.Range)
  Dim rDest As Word.Range: Set rDest = target.Duplicate
  Call rDest.MoveEnd(wdWord, IIf(rDest.Fields.Count = 1, -2, -1))
  
  Dim oldText$
  If Len(rDest.Text) < 30 Then
    oldText = rDest.Text
    Call rDest.Delete
    
    Call target.InsertBefore(Chr(11))
    
    Call target.InsertAfter(Chr(9))
    Call target.InsertAfter(oldText)
    Call target.InsertBefore(Chr(9))
    rDest.End = target.End
    rDest.Start = rDest.End
    Call rDest.MoveStart(wdWord, -1)
    Call rDest.Delete
  Else
    Dim rSecondLine As Word.Range: Set rSecondLine = rDest.Duplicate
    Call rSecondLine.Collapse(wdCollapseEnd)
    Do
      Call rDest.MoveEnd(wdWord, -1)
      Call rSecondLine.MoveStart(wdWord, -1)
    Loop Until Len(rDest.Text) + 2 < Len(rSecondLine.Text) Or rDest.Text = "     "
    
    Call target.InsertBefore(Chr(9))
    
    oldText = rSecondLine.Text
    Call rSecondLine.Delete
    Call target.InsertAfter(Chr(9))
    Call target.InsertAfter(oldText)
    Call rDest.InsertAfter(Chr(11))
  End If
End Function

Private Function CleanFooter(target As Word.Range, cPos As ColontitlePosition)
  target.Style = " "
  
  Dim hasField As Boolean: hasField = target.Fields.Count > 0
  If hasField Then
    Call target.Fields(1).Cut
    Call target.Delete
    Call target.Paste
  Else
    Call target.Delete
  End If
  
  If Not cPos.left_ And Not cPos.top_ Then _
    Call target.InsertBefore(Chr(9))
End Function

Private Function HeadersWrap(target As Word.Range)
' 
  Dim temprang1 As Word.Range, temprang2 As Word.Range
  If Len(target.Text) < 70 Then
    Call target.InsertBefore(Chr(11))
  Else
    Set temprang2 = target.Duplicate
    Set temprang1 = target.Duplicate
    temprang1.End = temprang2.Start
    Do
      Call temprang2.MoveStart(wdWord, 1)
      Call temprang1.MoveEnd(wdWord, 1)
    Loop Until Len(temprang1.Text) > Len(temprang2.Text)
    
    Call temprang2.InsertBefore(Chr(11))
  End If
End Function

Private Function GetHeader(target As Word.Section, cPos As ColontitlePosition) As Word.HeaderFooter
  Dim colID As WdHeaderFooterIndex: colID = IIf(cPos.left_, wdHeaderFooterEvenPages, wdHeaderFooterPrimary)
  If cPos.top_ Then
    Set GetHeader = target.Headers(colID)
  Else
    Set GetHeader = target.Footers(colID)
  End If
End Function

Private Function DoLink(sMode As TSource, isNew As Boolean) As Boolean
  DoLink = (sMode <> T_SOURCE_SECTION And sMode <> T_SOURCE_CHAPTER) Or (sMode = T_SOURCE_CHAPTER And isNew)
End Function

Private Function IsUsualSecName(sName$, target As Word.Range) As Boolean
  IsUsualSecName = Not (ContainsSubstr(target, "") + ContainsSubstr(target, "") _
  + ContainsSubstr(target, "") + IsToCHeader(sName))
End Function

Function ContainsSubstr(target As Word.Range, subText$) As Boolean
  ContainsSubstr = False
  If target.Words.Count < 2 Then _
    Exit Function
  ContainsSubstr = InStr(UCase(target.Words.First), UCase(subText)) <> 0
End Function

Private Function IsToCHeader(ByVal secName$) As Boolean
  secName = UCase(secName)
  IsToCHeader = InStr(secName, "") + InStr(secName, "")
End Function
