Attribute VB_Name = "CD_RedesignFonts"
Option Private Module
Option Explicit

Public Function ExecuteRedesign(target As Word.Document, props As ItemFontScale)
  target.Styles(BASE_STYLE).ParagraphFormat.LineSpacing = props.lineSpacing_
  target.Styles(BASE_STYLE).Font.Size = props.textSize_
  
  Call CSE_ProgressBar.Init(" ", maxVal:=3)
  Call CSE_ProgressBar.ShowModeless
  
  Call RedesignHeader(target, props)
  Call CSE_ProgressBar.IncrementA
  
  Call RedesignRegularStyles(target, props)
  Call CSE_ProgressBar.IncrementA
  
  Call RedesignText(target, props)
  Call CSE_ProgressBar.IncrementA
  
  Unload CSE_ProgressBar
End Function

' ===========
Private Function RedesignHeader(target As Word.Document, props As ItemFontScale)
  CSE_ProgressBar.Header = " "
  Call CSE_ProgressBar.InitSecondBar(0, target.Styles.Count)
  Dim aStyle As Word.Style
  For Each aStyle In target.Styles
    If Not IsStyleMajor(aStyle) Then _
      GoTo NEXT_STYLE
    
    aStyle.Font.Size = aStyle.Font.Size * props.fontMultiplier_
    
    If Not RequiresSpacingFix(aStyle) Then _
      GoTo NEXT_STYLE
    
    Dim oldScale As Double: oldScale = GetStyleScale(aStyle, props.fontFactor_)
    With aStyle.ParagraphFormat
      .LineSpacing = .LineSpacing * props.fontMultiplier_ * props.spacing_ / oldScale
      .SpaceAfter = .SpaceAfter * props.fontMultiplier_ * props.spacing_ / oldScale
      .SpaceBefore = .SpaceBefore * props.fontMultiplier_ * props.spacing_ / oldScale
    End With
NEXT_STYLE:
    Call CSE_ProgressBar.IncrementB
  Next aStyle
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function RedesignRegularStyles(target As Word.Document, props As ItemFontScale)
  CSE_ProgressBar.Header = "  "
  Call CSE_ProgressBar.InitSecondBar(0, target.Styles.Count)
  
  Dim aStyle As Word.Style
  For Each aStyle In target.Styles
    If Not RequiresRegularFix(aStyle) Then _
      GoTo NEXT_STYLE
    With aStyle.ParagraphFormat
      .SpaceAfter = .SpaceAfter * props.fontMultiplier_ * props.spacing_
      .SpaceBefore = .SpaceBefore * props.fontMultiplier_ * props.spacing_
    End With
NEXT_STYLE:
    Call CSE_ProgressBar.IncrementB
  Next aStyle
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function RedesignText(target As Word.Document, props As ItemFontScale)
  CSE_ProgressBar.Header = " "
  Call CSE_ProgressBar.InitSecondBar(0, target.ListParagraphs.Count)
  
  Dim para As Word.Paragraph
  For Each para In target.ListParagraphs
    If Not IsHeader(para.Range) And para.SpaceAfter <> 0 Then _
      para.SpaceAfter = props.lineSpacing_
    CSE_ProgressBar.IncrementB
  Next para
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function IsStyleMajor(aStyle As Word.Style) As Boolean
  IsStyleMajor = _
    aStyle.NameLocal Like "[]*" Or _
    aStyle.NameLocal Like "[] []" Or _
    aStyle.NameLocal Like "[] " Or _
    aStyle.NameLocal Like "[] #" Or _
    aStyle.NameLocal Like "[]*"
End Function

Private Function RequiresSpacingFix(aStyle As Word.Style)
  RequiresSpacingFix = _
    aStyle.NameLocal Like "[] #" Or _
    aStyle.NameLocal Like "[]*"
End Function

Private Function RequiresRegularFix(aStyle As Word.Style)
  RequiresRegularFix = _
    UCase(aStyle.NameLocal) Like "! " Or _
    UCase(aStyle.NameLocal) Like "!  " Or _
    UCase(aStyle.NameLocal) Like ""
End Function

Private Function GetStyleScale(aStyle As Word.Style, fontFactor As Double) As Double
  Dim spacingType As WdLineSpacing: spacingType = aStyle.ParagraphFormat.LineSpacingRule
  Select Case (spacingType)
  Case wdLineSpace1pt5: GetStyleScale = 1.5
  Case wdLineSpaceDouble: GetStyleScale = 2
  Case wdLineSpaceMultiple: GetStyleScale = aStyle.ParagraphFormat.LineSpacing
  Case wdLineSpaceExactly: GetStyleScale = aStyle.ParagraphFormat.LineSpacing / aStyle.Font.Size / fontFactor
  Case Else: GetStyleScale = 1
  End Select
End Function
