Attribute VB_Name = "Main"
Option Explicit

Public Sub CDA_AutoDesign()
  Call CDD_AutoDesign.Show
  If Not CDD_AutoDesign.isCancelled_ Then _
    Call RunAutoDesign(CDD_AutoDesign.Preferences)
    
  Call Unload(CDD_AutoDesign)
  Call UserInteraction.ShowMessage(IM_AUTODESIGN_OK)
End Sub

Public Sub CDA_FixLines()
  Dim theDoc As Word.Document:  Set theDoc = ActiveDocument
  
  Call CSE_ProgressBar.Init("", " ...", maxVal:=3)
  Call CSE_ProgressBar.ShowModeless
  
  '   ,   
  Call UpdateListsLayout(theDoc)
  Call CSE_ProgressBar.IncrementA
  
  Call UpdateTextLayout(theDoc)
  Call CSE_ProgressBar.IncrementA
  
  '    
  Call UpdateObjectFields(theDoc)
  Call CSE_ProgressBar.IncrementA
  
  Call Unload(CSE_ProgressBar)
  Call UserInteraction.ShowMessage(IM_FIX_LINES_OK)
End Sub

Public Sub CDA_Repaint()
  Dim theDoc As Word.Document: Set theDoc = ActiveDocument
  
  Call CDD_Paint.Init
  Call CDD_Paint.Show
  If CDD_Paint.isCancelled_ Then _
    Exit Sub
  
  Dim wordUI As New API_WordWrapper: Call wordUI.SetDocument(theDoc)
  Call wordUI.PauseUI
  
  Call CSE_ProgressBar.Init("", maxVal:=CDD_Paint.Count)
  Call CSE_ProgressBar.ShowModeless
  
  If CDD_Paint.DoText Then
    Call RepaintText(theDoc)
    Call CSE_ProgressBar.IncrementA
  End If
  
  If CDD_Paint.DoFields Then
    Call RepaintTextShapes(theDoc)
    Call CSE_ProgressBar.IncrementA
  End If
  
  If CDD_Paint.DoToC Then
    Call RepaintToC(theDoc)
    Call CSE_ProgressBar.IncrementA
  End If
  
  If CDD_Paint.DoLinks Then
    Call RepaintHLinks(theDoc)
    Call CSE_ProgressBar.IncrementA
  End If
  
  Call Unload(CSE_ProgressBar)
  Call wordUI.ResumeUI
  Call UserInteraction.ShowMessage(IM_REPAINT_OK)
End Sub

Public Sub CDA_FooterHeader()
  Dim theDoc As Word.Document: Set theDoc = ActiveDocument
  Dim docMeta As New InfoDocument: Call docMeta.Init(theDoc)

  Call CDD_HeaderFooter.Init(theDoc, docMeta.IsValidNames)
  Call CDD_HeaderFooter.Show
  If CDD_HeaderFooter.isCancelled_ Then _
    Exit Sub
    
  Dim props As ItemColontitles: Set props = CDD_HeaderFooter.Data
  Call Unload(CDD_HeaderFooter)
  
  Call CSE_ProgressBar.Init(" ", sHeader:="...", maxVal:=props.finish_ - props.start_ + 1)
  Call CSE_ProgressBar.ShowModeless
  
  Call CreateColontitles(theDoc, docMeta, props, "IncrementA")
  
  Call Unload(CSE_ProgressBar)
  Call UserInteraction.ShowMessage(IM_FOOTER_OK)
End Sub

Public Sub CDA_InsertBreak()
  Dim theDoc As Word.Document:  Set theDoc = ActiveDocument
  Dim theRange As Word.Range:  Set theRange = theDoc.ActiveWindow.Selection.Range
  If Selection.Range.Start = Selection.Range.End Then
    Set theRange = Selection.Paragraphs(1).Range
  End If
  
  Call InsertOneColomnSection(theRange)
End Sub

Public Sub CDA_InsertHeader()
  Dim theDoc As Word.Document:  Set theDoc = ActiveDocument
  
  Dim sHeader$: sHeader = UserInteraction.PromptInput("  ")
  If sHeader = vbNullString Then _
    Exit Sub
  Dim nLevel&: nLevel = Int(UserInteraction.PromptInput("  "))
  If nLevel < 2 And nLevel > 5 Then _
    Exit Sub
  
  Dim theRange As Word.Range: Set theRange = theDoc.ActiveWindow.Selection.Range
  If theRange.Paragraphs.Count < 2 Then
    Set theRange = theRange.Paragraphs(1).Range
  End If
  
  Call InsertHeader(theRange, sHeader, nLevel)
End Sub

Public Sub CDA_UpdateToC()
  Dim theDoc As Word.Document:  Set theDoc = ActiveDocument
  
  Dim wordUI As New API_WordWrapper: Call wordUI.SetDocument(theDoc)
  Call wordUI.PauseUI
  
  Dim toc As Word.TableOfContents
  For Each toc In theDoc.TablesOfContents
    Call UpdateTableOfContents(toc)
    Call RepaintToC(toc)
  Next toc
  
  Call wordUI.ResumeUI
End Sub

Public Sub CDA_TSAlign()
  If ActiveDocument.ActiveWindow.Selection.Tables.Count <> 1 Then
    Call UserInteraction.ShowMessage(EM_TABLE_NOT_SELECTED)
    Exit Sub
  End If
  Call PrepareTableForSplit(ActiveDocument.ActiveWindow.Selection.Tables(1))
End Sub

Public Sub CDA_TSSlice()
  If ActiveDocument.ActiveWindow.Selection.Tables.Count <> 1 Then
    Call UserInteraction.ShowMessage(EM_TABLE_NOT_SELECTED)
    Exit Sub
  End If
  
  Dim target As Word.Table: Set target = ActiveDocument.ActiveWindow.Selection.Tables(1)
  If target.Rows(1).Cells.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_TABLE_MERGED_CELLS)
    Exit Sub
  End If
  
  Call RemoveSplitMarkerFrom(target)
  ActiveWindow.ActivePane.View.Zoom.PageFit = wdPageFitFullPage
  target.Rows.AllowBreakAcrossPages = False

  Call CSE_ProgressBar.Init(" ", maxVal:=2)
  Call CSE_ProgressBar.ShowModeless
  
  Dim wordUI As New API_WordWrapper: Call wordUI.SetDocument(ActiveDocument)
  Call wordUI.PauseUI
  
  CSE_ProgressBar.Description = " ..."
  Dim slices As New Collection: Set slices = SliceTable(target)
  Call CSE_ProgressBar.IncrementA
  
  CSE_ProgressBar.Description = " ..."
  Call SplitSlices(slices)
  Call CSE_ProgressBar.IncrementA
  
  With ActiveWindow.ActivePane.View.Zoom
    .PageColumns = 2
    .PageRows = 1
  End With
  
  Call wordUI.ResumeUI
  Call Unload(CSE_ProgressBar)
End Sub

Public Sub CDA_TSPaint()
'    
  Dim theDoc As Word.Document:  Set theDoc = ActiveDocument
  Dim target As Word.Range: Set target = theDoc.Application.Selection.Range
  If target.Tables.Count < 1 Then
    Call UserInteraction.ShowMessage(EM_TABLE_NOT_SELECTED)
    Exit Sub
  End If
  
  Call CDD_TablePrototype.Init(theDoc.Tables.Count)
  Call CDD_TablePrototype.Show
  If CDD_TablePrototype.isCancelled_ Then _
    Exit Sub
    
  Dim nPrototype&: nPrototype = CDD_TablePrototype.GetPickedID
  Dim applyToAll As Boolean: applyToAll = CDD_TablePrototype.applyToAll
  Call Unload(CDD_TablePrototype)
  
  If nPrototype = 0 Then _
    Exit Sub
    
  Call RunPaintTableProto(target, IIf(applyToAll, target.Tables.Count, 1), theDoc.Tables(nPrototype))
  Call UserInteraction.ShowMessage(IM_REPAINT_OK)
End Sub

Public Sub CDA_PaintTable()
  Dim theDoc As Word.Document: Set theDoc = ActiveDocument
  Dim rSelection As Word.Range: Set rSelection = theDoc.Application.Selection.Range
  Dim tableCount&: tableCount = rSelection.Tables.Count
  If rSelection.Tables.Count < 1 Then
    Call UserInteraction.ShowMessage(EM_TABLE_NOT_SELECTED)
    Exit Sub
  End If
  
  Dim headRange As Word.Range: Set headRange = GetSectionHeader(rSelection.Start, theDoc)
  If headRange Is Nothing Then
    Call UserInteraction.ShowMessage(EM_TABLE_OUTSIDE_SECTION)
    Exit Sub
  End If
  
  Call CDD_TableColors.Init(ColorGetRGB(headRange.Font.Color, theDoc))
  Call CDD_TableColors.Show
  If CDD_TableColors.isCancelled_ Then _
    Exit Sub
  Dim props As TablePaintProps: props = CDD_TableColors.TableProperties
  Call Unload(CDD_TableColors)
  
  Call RunPaintTable(rSelection, tableCount, props)
  
  Call UserInteraction.ShowMessage(IM_REPAINT_OK)
End Sub

Public Sub CDA_InlineFields()
  Dim theDoc As Word.Document: Set theDoc = ActiveDocument
  Dim aField As Word.Field
  For Each aField In theDoc.Fields
    If aField.Type = wdFieldRef Then
      aField.Copy
      Call aField.Select
      Call theDoc.Application.Selection.Range.PasteAndFormat(wdFormatPlainText)
    End If
  Next aField
End Sub

Public Sub CDA_InsertObject()
  Dim target As Word.Range: Set target = ActiveDocument.ActiveWindow.Selection.Range.Paragraphs.First.Range
  Call CDD_AddPict.Show
  Select Case CDD_AddPict.Flag
    Case 0: Call InsertConceptSymbol(target, CDD_AddPict.Choice)
    Case 1: Call InsertPictureRef(target, CDD_AddPict.Bookmark)
    Case 2: Call InsertNewPicture(CDD_AddPict.PicAdd, target)
    Case 3: Call InsertTextField(target, CDD_AddPict.LighterText)
  End Select
End Sub

Public Sub CDA_InlineObjects()
  Dim target As Word.Range: Set target = ActiveDocument.ActiveWindow.Selection.Paragraphs.First.Range.Duplicate
  Dim nShape&
  For nShape = 1 To target.InlineShapes.Count Step 1
    Call InlineAsPNG(target.InlineShapes(nShape))
  Next nShape
End Sub

Public Sub CDA_CreateBlocking()
  Dim target As Word.Range: Set target = ActiveDocument.ActiveWindow.Selection.Range
  If target.Start = target.End Then _
    Exit Sub
  Call CreateLayoutBlock(target)
End Sub

Public Sub CDA_ScaleFont()
  Dim theDoc As Word.Document: Set theDoc = ActiveDocument
  
  Call CDD_FontScaling.Init
  Call CDD_FontScaling.Show
  
  If Not CDD_FontScaling.isCancelled_ Then _
    Call ExecuteRedesign(theDoc, CDD_FontScaling.SumUp)
  
  Call Unload(CDD_FontScaling)
End Sub

Public Sub CDA_Help()
  MsgBox "TODO"
End Sub

' =======
Private Function RunPaintTableProto(target As Word.Range, tableCount&, proto As Word.Table)
  Call CSE_ProgressBar.Init(" ", sHeader:="...", maxVal:=tableCount, canInterrupt:=True)
  Call CSE_ProgressBar.ShowModeless
  
  Dim nTable&
  For nTable = 1 To tableCount Step 1
    Dim aTable As Word.Table: Set aTable = target.Tables(nTable)
    If aTable.Range.Start = proto.Range.Start Then _
      GoTo NEXT_TABLE
      
    Call PaintTableProto(aTable, proto)
    
NEXT_TABLE:
    Call CSE_ProgressBar.IncrementA
    If CSE_ProgressBar.Interrupted Then _
      nTable = tableCount
  Next nTable

  Call Unload(CSE_ProgressBar)
End Function

Private Function RunPaintTable(target As Word.Range, tableCount&, props As TablePaintProps)
  Dim tablCount&: tablCount = Selection.Tables.Count
  
  Call CSE_ProgressBar.Init(" ", sHeader:="  ...", maxVal:=tablCount, canInterrupt:=True)
  Call CSE_ProgressBar.ShowModeless
  
  Dim nTable&
  For nTable = 1 To tableCount Step 1
    Dim aTable As Word.Table: Set aTable = target.Tables(nTable)
    Dim headRange As Word.Range: Set headRange = GetSectionHeader(aTable.Range.Start, target.Document)
    If headRange Is Nothing Then _
      GoTo NEXT_TABLE
      
    Call PaintTable(aTable, headRange.Font.Color, props)
    
NEXT_TABLE:
    Call CSE_ProgressBar.IncrementA
    If CSE_ProgressBar.Interrupted Then _
      nTable = tableCount
  Next nTable
  
  Call Unload(CSE_ProgressBar)
End Function
